<?php

/*
 * This file is part of the Tajine software.
 * <https://gitlab.adullact.net/adullact/pki/tajine>
 *
 * Copyright (c) ADULLACT   <https://adullact.org>
 *               Association des Développeurs et Utilisateurs de Logiciels Libres
 *               pour les Administrations et les Collectivités Territoriales
 *
 * Tajine is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this software. If not, see <https://www.gnu.org/licenses/agpl-3.0.en.html>.
 */

declare(strict_types=1);

namespace Scheduled;

use App\Repository\CertificateRepository;
use App\Repository\UserNoticeRepository;
use App\Tests\Functional\TestHelperTrait;
use Symfony\Bundle\FrameworkBundle\KernelBrowser;
use Symfony\Bundle\FrameworkBundle\Test\WebTestCase;
use Symfony\Component\DomCrawler\Crawler;
use Symfony\Component\HttpFoundation\Response;

class ScheduledTestRemindCertificateEndOfLifeTest extends WebTestCase
{
    use TestHelperTrait;

    public const HTTP_REMIND_TOKEN = 'sysadmin@test.example.org';

    private $registry;
    private array $certificateInitialDates ;

    /**
     * This method is called before each test.
     *
     * - remove all UserNotices in database
     * - backup certificate date (ValidTo) for tearDown() method
     */
    protected function setUp(): void
    {
        $registry = self::getContainer()->get('Doctrine\Persistence\ManagerRegistry');

        $certificateRepository = new CertificateRepository($registry);
        $certificates = $certificateRepository->findAll();
        $this->certificateInitialDates = [];
        foreach ($certificates as $cert) {
            $this->certificateInitialDates[$cert->getId()] = $cert->getValidTo();
        }
        $this->cleanUpUserNoticeTable($registry);
        self::ensureKernelShutdown();
    }

    /**
     * This method is called after each test.
     *
     *  - remove all UserNotices in database
     *  - restore all certificate date (ValidTo) in database
     */
    protected function tearDown(): void
    {
        $registry = self::getContainer()->get('Doctrine\Persistence\ManagerRegistry');
        $certificateRepository = new CertificateRepository($registry);
        $certificates = $certificateRepository->findAll();
        foreach ($certificates as $cert) {
            $cert->setValidTo($this->certificateInitialDates[$cert->getId()]);
            $certificateRepository->save($cert, true);
        }
        $this->cleanUpUserNoticeTable($registry);
        self::ensureKernelShutdown();
    }

    /**
     * Remove all UserNotices in database
     */
    private function cleanUpUserNoticeTable($registry): void
    {
        $noticeRepository = new UserNoticeRepository($registry);
        $notices = $noticeRepository->findAll();
        foreach ($notices as $notice) {
            $noticeRepository->remove($notice, true);
        }
    }

    /**
     * Send HTTP request to load reminder
     */
    private function httpRemindLoad(KernelBrowser $client, $httpToken = self::HTTP_REMIND_TOKEN): Crawler
    {
        $crawler = $client->request('GET', "/poc/ping-reminder/$httpToken");
        $this->assertRouteSame('app_legacy_cron_certificateRemindLifetime');
        return $crawler;
    }

    /**
     * Send HTTP request to load reminder with valid token
     */
    public function testHttpRequestToLoadRemindWithValidToken(): void
    {
        $client = static::createClient();
        $crawler = $this->httpRemindLoad($client);
        $this->assertResponseStatusCodeSame(Response::HTTP_ACCEPTED); // HTTP status code = 202
        $this->assertSame(0, count($crawler->getIterator()));
    }

    /**
     * Send HTTP request to load reminder with invalid token
     */
    public function testTryHttpRequestToLoadRemindWithInvalidToken(): void
    {
        $httpToken = 'not-valid-token';
        $client = static::createClient();
        $crawler = $this->httpRemindLoad($client, 'not-valid-http-token');
        $this->assertResponseStatusCodeSame(Response::HTTP_NOT_FOUND); // HTTP status code = 404
        $this->assertSame('Error 404 - Not Found', $crawler->text());
    }

    public function testTryToSendRemindOnCertificatesThatAreNotAtEndOfLife(): void
    {
        $client = static::createClient();
        $this->registry = self::getContainer()->get('Doctrine\Persistence\ManagerRegistry');
        $certificateRepository = new CertificateRepository($this->registry);
        $noticeRepository = new UserNoticeRepository($this->registry);

        // Update all certificate dates so they don't expire in the next few months.
        $now = new \DateTimeImmutable();
        $noRemindDate = $now->add(\DateInterval::createFromDateString("90 day"));
        $certificates = $certificateRepository->findAll();
        foreach ($certificates as $cert) {
//          dump($cert->getValidTo());
            $cert->setValidTo($noRemindDate);
            $certificateRepository->save($cert, true);
        }

        // Load reminder
        $crawler = $this->httpRemindLoad($client);
        $this->assertResponseStatusCodeSame(Response::HTTP_ACCEPTED); // HTTP status code = 202

        // No reminder notice stored in db
        $notices = $noticeRepository->findAll();
        $this->assertSame(0, count($notices));

        # No EMAIL sent
        $this->assertNull($this->getMailerEvent(0));
    }

    public function testSendRemindOnCertificatesThatAreAtEndOfLife(): void
    {
        $client = static::createClient();
        $this->registry = self::getContainer()->get('Doctrine\Persistence\ManagerRegistry');
        $certificateRepository = new CertificateRepository($this->registry);
        $noticeRepository = new UserNoticeRepository($this->registry);

        // Update all certificate dates so that they expire soon.
        $now = new \DateTimeImmutable();
        $certificates = $certificateRepository->findAll();
        $maxDay = 30;
        foreach ($certificates as $cert) {
//          dump($cert->getValidTo());
            $numberOfDays = $maxDay - rand(0, $maxDay);
            $noRemindDate = $now->add(\DateInterval::createFromDateString("$numberOfDays day"));
            $cert->setValidTo($noRemindDate);
            $certificateRepository->save($cert, true);
        }

        // Load reminder
        $crawler = $this->httpRemindLoad($client);
        $this->assertResponseStatusCodeSame(Response::HTTP_ACCEPTED); // HTTP status code = 202
        $expectedEmailsSent = count($certificates) * 2;

        // Reminder notices stored in db (certificates x 2)
        $expectedNotice = $expectedEmailsSent;
        $notices = $noticeRepository->findAll();
        $this->assertSame($expectedEmailsSent, count($notices));

        # EMAIL sent (certificates x 2)
        $this->assertNotNull($this->getMailerEvent(0));
        $this->assertEmailIsQueued($this->getMailerEvent(0));
        $this->assertEmailCount($expectedEmailsSent); // use assertQueuedEmailCount() when using Messenger

        // Reload reminder  ------------------------------------
        $crawler = $this->httpRemindLoad($client);
        $this->assertResponseStatusCodeSame(Response::HTTP_ACCEPTED); // HTTP status code = 202

        // Reminder notice stored in db
        $notices = $noticeRepository->findAll();
        $this->assertSame($expectedNotice, count($notices));

        # No EMAIL sent
        $this->assertNull($this->getMailerEvent(0));
    }
}
